
# Reverter transformações -------------------------------------------------

# Primeiro revertemos diferença dos dados ajustados e pontos de previsão
# de todos os modelos, com finalidade de obter valores em nível e poder
# comparar com expectativas Focus/BCB

# Valores ajustados (in-sample)
fitted_baseline <- fit_baseline %>%
  stats::fitted() %>%
  dplyr::as_tibble() %>%
  tidyr::drop_na()

# Pontos de previsão (out-of-sample)
fc_baseline <- frcst_baseline %>%
  dplyr::as_tibble() %>%
  dplyr::select("date", ".id", "h", ".model", "frcst" = ".mean")

# Juntar valores ajustados e pontos de previsão e reverter diferença
diff_rev_baseline <- dplyr::full_join(
  x = fitted_baseline,
  y = fc_baseline,
  by = c(".id", ".model", "date")
  ) %>%
  dplyr::mutate(value = dplyr::coalesce(.fitted, frcst)) %>%
  dplyr::arrange(.id, .model, .data$date) %>%
  dplyr::group_by(.id, .model) %>%
  # Converte para lista onde cada elemento é uma interação .id x .model
  dplyr::group_map(~.x, .keep = TRUE) %>%
  # Adiciona ao início das séries as observações necessárias para reverter diff
  purrr::map_df(
    .f = ~{

      # Salva informação da 1ª observação (ajuste) do modelo
      date_min <- .x %>%
        dplyr::filter(date == min(.data$date)) %>%
        dplyr::pull(.data$date)

      # Filtra 1ª observação do IDP
      data_add <- df_idp %>%
        dplyr::filter(date == min(.data$date)) %>%
        dplyr::select("date", "value" = "idp") %>%
        dplyr::mutate(
          date = tsibble::yearmonth(.data$date)
          )

      # Filtra entre 1ª observação do IDP e 1ª observação (ajuste) do modelo
      # possível gap e preenche com dados do objeto dataset_idp/df_idp
      if(unique(.x$.model) %in% c("rw", "rw_drift")){
        data_gap <- dplyr::mutate(data_add, date = .data$date + 1)
        } else {
          data_gap <- dataset_idp %>%
            dplyr::filter(
              date > dplyr::pull(data_add, .data$date) & date < date_min
              ) %>%
            dplyr::select("date", "value" = "idp")
        }

      if(unique(.x$.model) %in% c("rw", "rw_drift")){data_add = NULL}

      # Reverte diferença
      .x %>%
        dplyr::add_row(data_add) %>%
        dplyr::add_row(data_gap) %>%
        dplyr::arrange(.data$date) %>%
        dplyr::mutate(value = cumsum(value))
      }
    ) %>%
  tidyr::drop_na(h)

# Recalcular RMSE usando pontos de previsão em nível
rmse_baseline <- diff_rev_baseline %>%
  dplyr::mutate(date = lubridate::as_date(.data$date)) %>%
  dplyr::inner_join(
    dplyr::select(df_idp, "date", "actual" = "idp"),
    by = "date"
    ) %>%
  dplyr::group_by(h, .model) %>%
  dplyr::summarise(
    rmse    = sqrt(mean((actual - value)^2, na.rm = TRUE)),
    .groups = "drop"
  )


# Previsão do Focus/BCB ---------------------------------------------------


# Coletar dados de Expectativas do Focus/BCB - IDP
raw_focus <- rbcb::get_market_expectations(
  type       = "annual",
  indic      = "Investimento direto no pa\u00eds",
  start_date = "2000-01-01"
  )

# Tratar dados
# Objetivo: obter expectativa em t para t+i, onde i = 1, 2, ..., 24 (médias mensais)
focus <- raw_focus %>%
  # Estatísticas dos últimos 30 dias
  dplyr::filter(baseCalculo == 0) %>%
  dplyr::select(
    "date_focus" = "Data",
    "horizon_my" = "DataReferencia",
    "fcst"       = "Mediana"
    ) %>%
  dplyr::arrange(.data$date_focus) %>%
  dplyr::mutate(
    horizon_ymd = lubridate::dmy(paste0("01-12-", .data$horizon_my)),
    # Obter nº de meses entre data do Focus e horizonte de previsão
    diff_months = lubridate::interval(.data$date_focus, .data$horizon_ymd) %/% months(1)
  ) %>%
  # Filtrar horizonte de previsão do Focus até 24 meses à frente
  dplyr::filter(!diff_months > 24 & !diff_months < 1) %>%
  dplyr::mutate(diff_months = paste0("t_", .data$diff_months)) %>%
  dplyr::group_by(.data$horizon_ymd, .data$diff_months) %>%
  # Obter média das expectativas por horizonte de previsão
  dplyr::summarise(fcst = mean(.data$fcst, na.rm = TRUE), .groups = "drop") %>%
  tidyr::pivot_wider(
    id_cols     = .data$horizon_ymd,
    names_from  = .data$diff_months,
    values_from = .data$fcst
  ) %>%
  # Filtrar série para remover NAs do início e fim
  tidyr::drop_na() %>%
  dplyr::arrange(horizon_ymd) %>%
  tidyr::pivot_longer(
    cols      = -"horizon_ymd",
    names_to  = "h",
    values_to = "fcst"
    ) %>%
  dplyr::mutate(
    h    = stringr::str_remove_all(h, "t_"),
    fcst = fcst * 1000 # converte de bilhão p/ milhão
    ) %>%
  dplyr::rename("date" = "horizon_ymd")


# Calcular RMSE
rmse_focus <- focus %>%
  dplyr::inner_join(
    dplyr::select(df_idp, "date", "actual" = "idp"),
    by = "date"
  ) %>%
  dplyr::group_by(h) %>%
  dplyr::summarise(
    rmse    = sqrt(mean((actual - fcst)^2, na.rm = TRUE)),
    .groups = "drop"
  ) %>%
  dplyr::mutate(h = as.integer(h), .model = "Focus")


# Salvar dados de acurácia (RMSE) dos modelos em nível
# readr::write_rds(dplyr::bind_rows(rmse_baseline, rmse_focus), "data/acc_baseline.rds")
