
# Importar dados ----------------------------------------------------------

# Ler metadados/códigos de coleta
metadata <- readr::read_rds(
  file = paste0(getwd(), "/metadados/metadados.rds")
  )

# Links para extração de dados da Secex
url_secex <- list(
  "cgce" = "https://balanca.economia.gov.br/balanca/IPQ/arquivos/Dados_cgce_mensal.csv",
  "totais" = "https://balanca.economia.gov.br/balanca/IPQ/arquivos/Dados_totais_mensal.csv"
  )

# Data inicial da coleta de dados
init_date <- as.Date("2003-01-01")

# Códigos de coleta de dados do SGS/BCB
codes_bcb <- metadata %>%
  dplyr::filter(fonte_direta == "BCB") %>%
  dplyr::summarise(
    codes = purrr::set_names(x = as.numeric(code), nm = acronimo)
  ) %>%
  purrr::flatten_dbl()

# Coleta de dados do SGS/BCB
raw_bcb <- GetBCBData::gbcbd_get_series(
  id          = codes_bcb,
  first.date  = init_date,
  use.memoise = FALSE
  )

# Baixar dados de Secex
if(!dir.exists("data")){dir.create("data")}
purrr::walk2(
  .x = url_secex,
  .y = names(url_secex),
  .f = ~download.file(
    url      = .x,
    destfile = paste0("./data/", .y, ".", tools::file_ext(.x)),
    mode     = "wb"
    )
  )

# Importar CSV: Grandes Categoria Econômicas
raw_secex_cgce <- readr::read_csv2(
  file   = "./data/cgce.csv",
  locale = readr::locale("pt", encoding = "latin1")
  )

# # Importar CSV: Índices de quantum e preço
raw_secex_totais <- readr::read_csv2(
  file   = "./data/totais.csv",
  locale = readr::locale("pt", encoding = "latin1")
  )



# Tratamentos de dados ----------------------------------------------------

# Limpar dados do BCB
bcb <- raw_bcb %>%
  dplyr::select(
    "date"     = "ref.date",
    "variable" = "series.name",
    "value"
    ) %>%
  tidyr::pivot_wider(
    id_cols     = "date",
    names_from  = "variable",
    values_from = "value"
    )

# Limpar dados Secex: Grandes Categoria Econômicas
df_secex_cgce <- raw_secex_cgce %>%
  dplyr::mutate(
    date = lubridate::make_date(year = `CO_ANO`, month = `CO_MES`, day = 1L),
    dplyr::across(where(is.character), ~stringr::str_to_lower(.x, locale = "br"))
  ) %>%
  dplyr::filter(
    `TIPO` == "exp",
    `TIPO_INDICE` == "preco",
    stringr::str_detect(string = `NO_CLASSIFICACAO`, pattern = "consumo")
  ) %>%
  dplyr::select("date", "exp_bc" = "INDICE") %>%
  dplyr::as_tibble()


# Limpar dados Secex: Índices de quantum e preço
df_secex_totais <- raw_secex_totais %>%
  dplyr::mutate(
    date = lubridate::make_date(year = `CO_ANO`, month = `CO_MES`, day = 1L),
    dplyr::across(where(is.character), ~stringr::str_to_lower(.x, locale = "br"))
  ) %>%
  dplyr::filter(
    `TIPO` %in% c("exp", "imp"),
    `TIPO_INDICE` %in% c("preco", "quantum")
  ) %>%
  tidyr::pivot_wider(
    id_cols     = "date",
    names_from  = c("TIPO", "TIPO_INDICE"),
    values_from = "INDICE"
  ) %>%
  dplyr::as_tibble()

# Juntar bases dados em um único tibble, preencher NAs e calcular variáveis
df_idp <- purrr::reduce(
  .x = list(bcb, df_secex_cgce, df_secex_totais),
  .f = dplyr::left_join,
  by = "date"
  ) %>%
  dplyr::select(dplyr::all_of(c("date", metadata$acronimo))) %>%
  dplyr::relocate("date", "idp") %>%
  dplyr::filter(
    date <= bcb %>%
      dplyr::select("date", "idp") %>%
      tidyr::drop_na() %>%
      dplyr::filter(date == max(.data$date)) %>%
      dplyr::pull(.data$date)
    ) %>%
  timetk::tk_augment_lags(          # cria lags 1:12 de cada variável
    .value = -.data$date,
    .lags  = 1:12
  ) %>%
  tidyr::fill(dplyr::everything(), .direction = "updown")

# Salvar dados para reprodução
# readr::write_rds(df_idp, "data/df_idp.rds")


# Análise de Componentes Principais ---------------------------------------

# Aplica Análise de Componentes Principais
pca_idp <- stats::prcomp(x = df_idp[-c(1:2)], scale. = TRUE)

# Nº de Componentes Principais com pelo menos 80% de variância % acumulada
pca_80 <- pca_idp %>%
  summary() %>%
  magrittr::extract2("importance") %>%
  magrittr::extract("Cumulative Proportion", ) %>%
  {min(which(. >= 0.8))}

# Salvar componentes
df_pca <- df_idp %>%
  dplyr::select("date", "idp") %>%
  dplyr::bind_cols(
    pca_idp %>%
      purrr::pluck("x") %>%
      dplyr::as_tibble() %>%
      dplyr::select(dplyr::all_of(1:pca_80))
  )


# Estacionariedade --------------------------------------------------------

# Obter nº de diferenças p/ série ser estacionária em formato tabular
vars_ndiffs <- df_pca %>%
  dplyr::select(-.data$date) %>%
  report_ndiffs()

# Aplicar diferenças
dataset_idp <- df_pca %>%
  dplyr::select(-.data$date) %>%
  purrr::map2(
    .y = vars_ndiffs$ndiffs,     # acrescentar número de difereças junto com a série
    .f = ~list(.x, .y)
  ) %>%
  purrr::keep(~.x[[2]] > 0) %>%  # manter séries que precisam ser diferenciadas
  purrr::map(                    # diferenciar séries de acordo com ndiffs
    .f = ~{                      # aplicar diferenças
      tsibble::difference(.x[[1]], lag = 1, differences = .x[[2]])
    }
  ) %>%
  purrr::list_modify(            # adiciona de volta séries que são estacionárias
    !!!df_pca %>%
      dplyr::select("date", vars_ndiffs$variable[vars_ndiffs$ndiffs == 0])
  ) %>%
  dplyr::as_tibble() %>%
  dplyr::relocate(.data$date, .data$idp) %>%
  tidyr::drop_na() %>%
  dplyr::mutate(date = tsibble::yearmonth(.data$date)) %>%
  tsibble::as_tsibble(index = "date")

